#include "pendulum_parametric.h"
#include "TH1D.h"
#include "TH2D.h"

using namespace std;
int main()
{
    //b) Solving the pendulum, graphs and modultaing depth
    pendulum_parametric Projectb(0.7, 0.2, {0.1, 0}); //Call constructor
    vector<ODEpoint> DimSol;                          //Vectors to store solutions
    vector<ODEpoint> AdimSol;
    double Time = 15 * 2 * M_PI * sqrt(1 / 9.8); //Run 15 periods
    Projectb.RungeKutta4(1E-2, Time);            //Run Runge Kutta 4 Method
    DimSol = Projectb.GetDimSolution();          //Return Dimensionalized Solution
    vector<double> Peaks;                        //Vector to store peak values

    for (int i = 0; i < DimSol.size(); ++i)
    {
        if (i != 0 && i != (DimSol.size() - 1))
        {
            if (fabs(DimSol[i][0]) > fabs(DimSol[i + 1][0]) && fabs(DimSol[i][0]) > fabs(DimSol[i - 1][0]))
            {
                Peaks.push_back(fabs(DimSol[i][0])); //Get Peak value if it is a local minimum/maximum
            }
        }
    }
    Projectb.Draw(1, Time);      //Draw the dimensionalized graph
    Projectb.Draw(0, Time);      //Draw the adimensionalized graph
    double Amin = 100, Amax = 0; // Helpers to get the maximum and minimum of the amplitude
    for (auto v : Peaks)
    {
        if (v > Amax)
        {
            Amax = v;
        }
        if (v < Amin)
        {
            Amin = v;
        }
    }
    //Print minimum maximum and medium amplitude
    cout << "\n-------------------------------------------------------------\n"<<endl;
    cout << "\033[1;32m ->Valor minimo do módulo da amplitude: \033[0m" << Amin << "\n\033[1;32m ->Valor maximo do módulo da amplitude: \033[0m" << Amax << "\n\033[1;32m ->Valor medio do módulo da amplitude: \033[0m" << (Amax + Amin) / 2 << endl;
    //Print modulating depth
    cout << "\n\033[1;35m ->Profundidade de modulacao: \033[0m" << (Amax - Amin) / (Amax + Amin) << endl;
    cout << "\n-------------------------------------------------------------\n\n"<<endl;
    AdimSol.clear();
    DimSol.clear();
    Peaks.clear(); //Clear Memory


    //c) Stability and moving average
    pendulum_parametric Projectc1(0.4, 0.2, {0.1, 0}); //Create two more objects with new values of delta and epsilon
    pendulum_parametric Projectc2(0.4, 0.36, {0.1, 0});
    Projectc2.RungeKutta4(1E-02, Time); //Solve the EDO with RK4
    Projectc2.DrawPhaseDiagram(0);      //Draw the adimensionalized Phase Diagram
    Projectc2.MovingAverage(4 * M_PI);  //Calculate the moving average with tw=4*PI
    Projectc2.DrawAverageGraph();       //Draw the graph of the moving average with the absolute value of theta hat
    Projectc1.RungeKutta4(1E-02, Time); //Repeat for the other value
    Projectc1.DrawPhaseDiagram(0);
    Projectc1.MovingAverage(4 * M_PI);
    Projectc1.DrawAverageGraph();

    //Drawing the stability Map
    double const deltai = 0.02, epsiloni = 0, grstep = 0.005, deltaend = 1.2,
                 epsilonend = 0.5, deltabin = (deltaend - deltai) / grstep + 1.,
                 epsilonbin = (epsilonend - epsiloni) / grstep + 1.;

    TH2D *h2 = new TH2D("h2", "Mapa dos pontos (#delta, #varepsilon) e estabilidade do sistema;#delta;#varepsilon",
                        ceil(deltabin), deltai, deltaend, ceil(epsilonbin), epsiloni, epsilonend);

    TH2D *h1 = new TH2D("h1", "Mapa dos pontos (#delta, #varepsilon) e estabilidade do sistema;#delta;#varepsilon", 
                        ceil(deltabin), deltai, deltaend, ceil(epsilonbin), epsiloni, epsilonend); //Create Histograms for stable and instable points

    double epsilon, delta;
    for (int i = 0; i < ceil(epsilonbin); ++i)
    {
        epsilon = epsiloni + i * grstep;
        for (int k = 0; k < ceil(deltabin); ++k)
        {
            delta = deltai + k * grstep;
            pendulum_parametric ProjectGrid(delta, epsilon, {0.1, 0}); //Create object with deltai and epsilon i
            ProjectGrid.RungeKutta4(1E-01, 2*Time); //Solve the new EDO
            ProjectGrid.MovingAverage(2 * M_PI); //Calculate the moving average
            if (ProjectGrid.CheckStability()) //Check Stability and fill accordingly 
                h2->Fill(delta, epsilon, 0.9); 
            else
                h1->Fill(delta, epsilon, 0.1);
        }
    }
    gStyle->SetPalette(kRedBlue); //Set Color Palette 
    TCanvas *c2 = new TCanvas("c2", "Stability", 0, 0, 1000, 500); //Create Canvas
    auto color2 = TColor::GetColor("#76373A"); //Get Color for label 
    auto color1 = TColor::GetColor("#1B3B74");
    c2->SetRightMargin(0.15); //Set Canvas Margins
    c2->SetLeftMargin(0.15);
    h2->SetStats(0);
    h2->SetFillColor(color1); //Set Fill Colors
    h1->SetFillColor(color2);
    h2->Draw("col");    //Draw histograms
    h1->Draw("same col");
    auto legend = new TLegend(0.70, 0.8, 0.85, 0.9); //Create label
    legend->SetHeader("Legenda", "C"); 
    legend->AddEntry(h2, "Pontos Estaveis", "f");//Add Entries
    legend->AddEntry(h1, "Pontos Instaveis", "f");
    legend->Draw();
    c2->Modified();
    c2->Update();
    c2->SaveAs("Grid.png"); //Update and save canvas
    c2->Clear();
    delete c2, legend, h1, h2; //Clear Memory
    TCanvas *c4 = new TCanvas("c1", "Pendulum", 100, 100, 900, 500); //Create Canvas
    vector<double> DeltaPontosEstaveis, EpsilonPontosEstaveis, DeltaPontosInstaveis, EpsilonPontosInstaveis; //Vectors for storing values
    double deltaaa = 0, epsilonnn = 0; //Helping variables for theoretical graph
    for (int i = 0; i <= 100; ++i)
    {
        epsilonnn = i * grstep;
        for (int k = 0; k <= 236; ++k)
        {
            deltaaa = 0.02 + k * grstep;
            if (epsilonnn / 2 > sqrt(deltaaa) - 2 * deltaaa && epsilonnn / 2 > 2 * deltaaa - sqrt(deltaaa)) //Instability criteria 
            {
                DeltaPontosInstaveis.push_back(deltaaa); //Store instable points
                EpsilonPontosInstaveis.push_back(epsilonnn);
            }
            else
            {
                DeltaPontosEstaveis.push_back(deltaaa); //Store stable points
                EpsilonPontosEstaveis.push_back(epsilonnn);
            }
        }
    }
    double *GraphEstDelta = &DeltaPontosEstaveis[0]; //Pass the vectors to arrays
    double *GraphEstEps = &EpsilonPontosEstaveis[0];
    double *GraphInstDelta = &DeltaPontosInstaveis[0];
    double *GraphInstEps = &EpsilonPontosInstaveis[0];
    TGraph *grEst = new TGraph(DeltaPontosEstaveis.size(), GraphEstDelta, GraphEstEps); //Graph of stable points
    TGraph *grInst = new TGraph(DeltaPontosInstaveis.size(), GraphInstDelta, GraphInstEps); //Graph of instable points
    TLegend *label = new TLegend(0.70, 0.75, 0.9, 0.9); //Label
    label->SetHeader("Legenda", "C"); //Label settings
    label->AddEntry(grEst, "Pontos Estaveis", "p");
    label->AddEntry(grInst, "Pontos Instaveis", "p");
    grEst->SetMarkerColor(TColor::GetColor("#2d698f")); //Set Marker Color, size and style
    grEst->SetMarkerStyle(21);
    grEst->SetMarkerSize(0.5);
    grEst->GetXaxis()->SetLimits(0.02, 1.2); //Axis settings
    grEst->SetMaximum(0.5);
    grEst->SetTitle("Mapa dos pontos (#delta, #varepsilon) e estabilidade do sistema;#delta;#varepsilon"); //Set title 
    grInst->SetMarkerColor(kRed + 2); //Set Marker Color, size and style
    grInst->SetMarkerStyle(21);
    grInst->SetMarkerSize(.5);
    grEst->Draw("AP"); //Draw 
    grInst->Draw("Same P");
    label->Draw();
    c4->Modified();
    c4->Update();
    c4->SaveAs("GridTeor.png"); //Save
    delete c4, grEst, grInst, label;

    //d) Fourier Analysis
    Projectc2.RungeKutta4(1E-01, Time); //Recalculate solutions for smaller time step 
    Projectb.RungeKutta4(1E-01, Time);
    DimSol=Projectb.GetDimSolution(); //Get vector of one of the dimensional solutions
    vector<ODEpoint> Sold2 = Projectc2.GetDimSolution(); //Store the vector that contains the other dimensional solution
    double dts1 = DimSol[1].T() - DimSol[0].T();         //See time step for both solutions
    double dts2 = Sold2[1].T() - Sold2[0].T();
    double fs1 = 1 / dts1, fs2 = 1 / dts2;                 //See sampling frequency
    int max1, max2, N1 = DimSol.size(), N2 = Sold2.size(); //Store size of solutions
    double sumR, sumI; //Real part and imaginary part variables for sum
    if (N1 % 2 == 0)
        max1 = N1 / 2 - 1; //If the number is even the maximum value for k
    else
        max1 = (N1 - 1) / 2; //If the number is odd the maximum value for k

    if (N2 % 2 == 0)
        max2 = N2 / 2 - 1; //Repeat for the other pair of delta and epsilon
    else
        max2 = (N2 - 1) / 2;

    double *Amplitude1 = new double[max1 + 1]; //Array to store Amplitude and Frequency for plotting
    double *Freq1 = new double[max1 + 1];
    double *Amplitude2 = new double[max2 + 1];
    double *Freq2 = new double[max2 + 1];
    for (int k = 0; k <= max1; ++k) //Cycle for each frequency
    {
        sumR = 0;
        sumI = 0;
        Freq1[k] = k / (dts1 * N1); //Add frequency to the vector
        for (int j = 0.; j < N1; ++j)
        {
            sumR += (DimSol[j][0]) * cos((k * 2. * M_PI * j) / (N1)); //Calculate the discrete sum of the Fourier integral, for the real and imaginary parts
            sumI += (DimSol[j][0]) * sin((k * 2. * M_PI * j) / (N1));
        }
        Amplitude1[k] = (sqrt(sumI * sumI + sumR * sumR)); //Get the norm of that sum and store it
    }
    for (int k = 0; k <= max2; ++k) //Cycle for each frequency
    {
        sumR = 0;
        sumI = 0;
        Freq2[k] = k / (dts2 * N2); //Add frequency to the vector
        for (int j = 0.; j < N2; ++j)
        {
            sumR += (Sold2[j][0]) * cos((k * 2. * M_PI * j) / (N2)); //Calculate the discrete sum of the Fourier integral, for the real and imaginary parts
            sumI += (Sold2[j][0]) * sin((k * 2. * M_PI * j) / (N2));
        }
        Amplitude2[k] = (sqrt(sumI * sumI + sumR * sumR)); //Get the norm of that sum and store it
    }
    double *Bins1 = new double[max1 + 2]; //Create vector of bins to make our frequency values the center of our bins
    double *Bins2 = new double[max2 + 2];
    Bins1[0] = 0;
    for (int k = 1; k <= max1; ++k)
    {
        Bins1[k] = (Freq1[k - 1] + Freq1[k]) / 2; //Store the medium value
    }
    Bins1[max1 + 1] = Freq1[max1];

    //Repeat for the other set of values
    Bins2[0] = 0;
    for (int k = 1; k <= max2; ++k)
    {
        Bins2[k] = (Freq2[k] + Freq2[k - 1]) / 2;
    }
    Bins2[max2 + 1] = Freq2[max2];

    TCanvas *c3 = new TCanvas("c", "FourierHist", 100, 100, 1600, 1020); //Canvas and histogram for plorring
    TH1D *H1 = new TH1D("Frequencies", "Decomposicao de Fourier em frequencias: #delta=0.7 e #varepsilon=0.2; Frequencia(Hz); Amplitude(rad)", max1 + 1, 0, Freq1[max1]);
    for (int k = 0; k <= max1; ++k)
        H1->Fill(Freq1[k], Amplitude1[k]); //Fill with the 1st solution
    gStyle->SetOptStat(kFALSE); //Set style
    H1->SetBins(max1 + 1, Bins1); //Set bin definitions
    H1->SetLineColor(kRed + 1); //Set Line color
    H1->SetFillColor(kRed + 1); //Set Fill color
    H1->Draw("HIST");
    c3->Update();
    c3->Modified();
    c3->SaveAs("histograma_freqs1.png"); //Update and save Canvas
    c3->Clear(); //Clear for new Histogram, created next
    TH1D *H2 = new TH1D("Frequencies2", "Decomposicao de Fourier em frequencias: #delta=0.4 e #varepsilon=0.36; Frequencia(Hz); Amplitude(rad)", max2 + 1, 0, Freq2[max2]);
    for (int k = 0; k <= max2; ++k)
        H2->Fill(Freq2[k], Amplitude2[k]); //Store values for 2nd solution
    H2->SetBins(max2 + 1, Bins2);
    H2->SetLineColor(kBlue + 1); //Set Line color
    H2->SetFillColor(kBlue + 1); //Set Fill color
    H2->Draw("HIST"); //Draw histogram 
    c3->Update();
    c3->Modified();
    c3->SaveAs("histograma_freqs2.png"); //Update and save Canvas
    delete c3, H1, H2;
    delete[] Bins1, Bins2; //Clear Memory
    vector<pair<double, double>> DFT1, DFT2; //Vector of pairs for ordering
    DFT1.resize(max1 + 1); //Resize vectors
    DFT2.resize(max2 + 1);
    for (int k = 0; k <= max1; k++)
    {
        DFT1[k] = make_pair(Amplitude1[k], Freq1[k]); //Store values of amplitude and frequency
    }
    for (int k = 0; k <= max2; k++)
    {
        DFT2[k] = make_pair(Amplitude2[k], Freq2[k]); //Repeat
    }
    sort(DFT1.rbegin(), DFT1.rend()); //Sort, being a pair it sorts with the first element taken into consideration
    sort(DFT2.rbegin(), DFT2.rend());
    cout << "-------------------------------------------------------------------------------"<<endl;
    cout << "\n \033[1;35m ->As 5 frequências (f_k) mais importantes para delta= 0.7 e epsilon= 0.2 são: \033[0m" << endl;
    for (int k = 0; k < 5; k++) //Print the first five elements, in other words, the five biggest importances of the frequencies on our signal
    {
        cout << "\t \033[1;32m ->f_k: \033[0m" << DFT1[k].second << " \033[1;32m Hz\tcom amplitude |X(f_k)|: \033[0m" << DFT1[k].first << "\033[1;32m rad \033[0m" << endl;
    }
    cout << "\n-------------------------------------------------------------------------------"<<endl;
    cout << "\n \033[1;35m ->As 5 frequências (f_k) mais importantes para delta= 0.4 e epsilon= 0.36 são: \033[0m" << endl;
    for (int k = 0; k < 5; k++) //Repeat
    {
        cout << "\t \033[1;32m ->f_k: \033[0m" << DFT2[k].second << " \033[1;32m Hz\tcom amplitude |X(f_k)|: \033[0m" << DFT2[k].first << "\033[1;32m rad \033[0m" << endl;
    }
    cout << "\n-------------------------------------------------------------------------------\n"<<endl;
    delete[] Amplitude1, Amplitude2, Freq1, Freq2;
    DFT1.clear();
    DFT2.clear();
    Sold2.clear();
    DimSol.clear(); //Clear memory
    return 0;
}