#include "ODEpoint.h"

Xvar::Xvar(int n) // number of dependent variables
{
    x.resize(n);
}

Xvar::Xvar(std::vector<double> l) : x(l) { ; } // passing vector

Xvar::Xvar(const std::initializer_list<double> &v) : x(v) { ; } // using initializer list to build object: X({1,2})

Xvar::Xvar(const Xvar &c):x(c.x){;} // copy constructor

Xvar::~Xvar()
{
    x.clear();
}

Xvar &Xvar::operator=(const Xvar &c)
{
    if (this != &c)
    {
        x = c.x;
    }
    return *this;
}


Xvar Xvar::operator+(const Xvar &c)
{
    std::vector<double> x2(x);
    std::transform(x.begin(), x.end(), c.x.begin(), x2.begin(), std::plus<double>());
    Xvar X2(x2);
    return X2;
}

Xvar operator*(double k, const Xvar& c)
{
    std::vector<double> x2 = c.x;
    for (auto s : x2)
    {
        s *= k;
    }
    return Xvar(x2);
}

std::ostream &operator<<(std::ostream &os, const Xvar &c)
{
    int i = 1;
    for (auto& p : c.x)
    {
        os << "Variable number " << i << " has value " << p << std::endl;
        i++;
    }
    return os;
}

std::vector<double>& Xvar::X()
{
    return x;
}

double &Xvar::operator[](int i)
{
    return x[i];
}

//ODEpoint class//////////////////////////////////////////////////////////////////////////////////
ODEpoint::~ODEpoint(){;}

void ODEpoint::SetODEpoint(double t_, Xvar &p)
{
    t=t_;
    x=p.X();
}
void ODEpoint::SetODEpoint(double t_, const std::initializer_list<double> &v)
{
    t=t_;
    x=v;
}
void ODEpoint::SetODEpoint(double t_, std::vector<double> v)
{
    t=t_;
    x=v;
}