#ifndef __PENDULUM__
#define __PENDULUM__

#include "ODEpoint.h"
#include <map>
#include <functional>
#include <cmath>
#include <sstream>
#include "TCanvas.h"
#include "TGraph.h"
#include "TAxis.h"
#include "TLegend.h"
#include "TStyle.h"
class pendulum_parametric
{
public:
    pendulum_parametric();
    pendulum_parametric(double, double, double, double, double, const Xvar &); //if we want to choose other values for L, m, g
    pendulum_parametric(double, double, const Xvar &);
    pendulum_parametric(double, double, const std::initializer_list<double> &); // pendulum(10, {80, 0})
    // length in meters, angles in degrees, velocity degrees/sec
    pendulum_parametric(double, double, double, double);
    ~pendulum_parametric();
    void Init();//Initialize the functions

    // solvers
    void RungeKutta4(double step=1E-3, double Time=10);
    
    // Draw graphics
    void Draw(bool vardim, double tf);//if vardim=1 use DimSol
    void DrawAverageGraph();
    void DrawPhaseDiagram(bool); 
    void DrawStabilityDiagram();
    bool CheckStability();
    
    // getters
    std::vector<ODEpoint>& GetDimSolution();
    std::vector<ODEpoint>& GetAdimSolution();

    //pendulum stability
    void MovingAverage(double);

private:
    const double L, g, M; // length (m), gravity(m/s), mass(Kg)
    
    double epsilon, delta; //Epsilon and delta for equation
    
    Xvar X0;  // initial conditions: angle (rad), angular velocity (rad/s)
    
    // solutions
    std::vector<ODEpoint> DimSol; // Dimensional solution
    std::vector<ODEpoint> AdimSol; // Adimensional solution
    
    //Moving Average of the Adimensional solution
    std::vector<ODEpoint> AverageSol;

    std::function<double(ODEpoint &)> f[2]; // f_theta(t,theta,omega), f_omega(t,theta,omega)
};

#endif