#include "DataPoints.h"

DataPoints::DataPoints(const std::vector<std::pair<double, double>> &x)
{
    P = x;

    std::sort(P.begin(), P.end());
}

DataPoints::DataPoints(int n, double *x, double *y)
{
    for (int i = 0; i < n; ++i)
    {
        P.push_back(std::make_pair(x[i], y[i]));
    }

    std::sort(P.begin(), P.end());
}

DataPoints::DataPoints(const std::vector<double> &x, const std::vector<double> &y)
{
    int N = x.size();
    for (int i = 0; i < N; ++i)
    {

        P.push_back(std::make_pair(x[i], y[i]));
    }

    std::sort(P.begin(), P.end());
}
const std::vector<std::pair<double, double>> DataPoints::GetPoints()
{
    return P;
}

DataPoints::~DataPoints()
{
    std::cout << "DataPoints Destructor called" << std::endl;
    P.clear();
}

TGraph *DataPoints::GetGraph()
{
    int j = 0;
    double x[P.size()];
    double y[P.size()];
    for (auto p : P)
    {
        x[j] = p.first;
        y[j] = p.second;
        j = j + 1;
    }
    TGraph *gr1 = new TGraph(P.size(), x, y);
    return gr1;
}

void DataPoints::Draw()
{
    //Instance of the graph
    TGraph *gr = DataPoints::GetGraph();

    gr->SetTitle("Pendulum L=4.0 [m] | Initial Cond.: #theta_{0}= 1.4 [rad] | v_{0}= 0.0 [rad/sec];Time [sec];Angle(#theta) [rad]");

    //Make the plot esthetically better
    gr->SetMarkerColor(kBlue);
    gr->SetMarkerStyle(kFullCircle);

    //Oppening an interface where the graph will be shown
    //TApplication app("app", nullptr, nullptr);

    //The canvas on which we'll draw the graph
    TCanvas *c1 = new TCanvas("c1", "Grafico", 550, 100, 800, 600);

    //Drawing DataPoints
    gr->Draw("AP");

    c1->Modified();
    c1->Update();

    //Saving the graph in a file (could be png, or eps, or pdf)
    c1->SaveAs("HW3.png");

    //To end program after the app has benn closed
    //TRootCanvas *rc = (TRootCanvas *)c1->GetCanvasImp();
    //rc->Connect("CloseWindow()", "TApplication", gApplication, "Terminate()");

    //Running the application
    //app.Run();

    delete c1;
    delete gr;
}

//adicionei isto////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
std::ostream &operator<<(std::ostream &os, const DataPoints &D)
{
    int i = 1;
    for (std::vector<std::pair<double, double>>::const_iterator p = D.P.begin(); p != D.P.end(); p++)
    {
        os << "N= " << i << ": x= " << p->first << " | y= " << p->second << std::endl;
        i++;
    }

    return os;
}
