#include "DerivatorIntegrator.h"

IntegDeriv::IntegDeriv(Functor &function) : F(function) { ; }

double IntegDeriv::GoldenSearch(double xi, double xf, int intervals, bool D)
{
    double ratio = (sqrt(5) - 1) / 2;
    double max = 0;

    if (D == true)
    {

        for (int i = 0; i < intervals; ++i)
        {

            double xl = (xf - xi) / intervals * i;
            double xu = (xf - xi) / intervals * (i + 1);
            double d = (xu - xl) * ratio;
            double epsilon = 0.001;
            double x1 = xl + d;
            double x2 = xu - d;

            while (1)
            {

                if (this->IntegDeriv::SecondDerivativeFivePoint(x1) > this->IntegDeriv::SecondDerivativeFivePoint(x2))
                {
                    xl = x2;
                    x2 = x1;
                    x1 = xl + ratio * (xu - xl);
                }

                if (this->IntegDeriv::SecondDerivativeFivePoint(x1) <= this->IntegDeriv::SecondDerivativeFivePoint(x2))
                {
                    xu = x1;
                    x1 = x2;
                    x2 = xu - ratio * (xu - xl);
                }

                if (xu - xl < epsilon)
                {
                    break;
                }
            }
            if (i == 0)
                max = this->IntegDeriv::SecondDerivativeFivePoint((xu + xl) / 2);
            else if (this->IntegDeriv::SecondDerivativeFivePoint((xu + xl) / 2) > max)
                max = this->IntegDeriv::SecondDerivativeFivePoint((xu + xl) / 2);
        }

        return max;
    }

    if (D == false)
    {
        for (int i = 0; i < intervals; ++i)
        {

            double xl = (xf - xi) / intervals * i;
            double xu = (xf - xi) / intervals * (i + 1);
            double d = (xu - xl) * ratio;
            double epsilon = 0.001;
            double x1 = xl + d;
            double x2 = xu - d;

            while (1)
            {

                if (this->IntegDeriv::FourthDerivative(x1) > this->IntegDeriv::FourthDerivative(x2))
                {
                    xl = x2;
                    x2 = x1;
                    x1 = xl + ratio * (xu - xl);
                }

                if (this->IntegDeriv::FourthDerivative(x1) <= this->IntegDeriv::FourthDerivative(x2))
                {
                    xu = x1;
                    x1 = x2;
                    x2 = xu - ratio * (xu - xl);
                }

                if (xu - xl < epsilon)
                {
                    break;
                }
            }
            if (i == 0)
                max = this->IntegDeriv::FourthDerivative((xu + xl) / 2);
            else if (this->IntegDeriv::FourthDerivative((xu + xl) / 2) > max)
                max = this->IntegDeriv::FourthDerivative((xu + xl) / 2);
        }

        return max;
    }

    return 0;
}

void IntegDeriv::TrapezoidalRule(double xi, double xf, double &Integral, double &Error, double tol)
{

    double L = xf - xi;
    Integral = L * (F(xi) + F(xf)) / 2;
    int k = 1;
    double iteration = 1;

    while (1)
    {
        ++k;
        double sumf = 0;
        double aux = Integral;

        for (int i = 1; i <= iteration; ++i)
        {
            sumf = sumf + F(xi + (2 * i - 1) * L / (iteration * 2));
        }

        Integral = Integral / 2 + L / (iteration * 2) * sumf;
        iteration = iteration * 2;

        if (fabs(Integral - aux) < tol)
        {
            Error = fabs((xf - xi) * (xf - xi) * (xf - xi) * GoldenSearch(xi, xf, 100, 0) / (12 * iteration * iteration));
            break;
        }
    }
}

void IntegDeriv::SimpsonRule(double xi, double xf, double &Integral, double &Error, int nintervals)
{
    Integral = 0;
    double h = (xf - xi) / nintervals;

    if (nintervals % 2 == 0)
    {
        double sumf = F(xi);
        for (int i = 1; i <= nintervals - 2; i = i + 2)
        {
            sumf = sumf + 4 * F(xi + i * h) + 2 * F(xi + (i + 1) * h);
        }

        sumf = sumf + 4 * F(xf - h) + F(xf);
        Integral = (sumf * h / 3);
    }
    else
    {
        double sumf = F(xi);
        for (int i = 1; i <= nintervals - 5; i = i + 2)
        {
            sumf = sumf + 4 * F(xi + i * h) + 2 * F(xi + (i + 1) * h);
        }
        sumf = sumf + 4 * F(xf - 4 * h) + F(xf - 3 * h);
        Integral = (h / 3 * sumf + 3 * h / 8 * (F(xf - 3 * h) + 3 * F(xf - 2 * h) + 3 * F(xf - h) + F(xf)));
    }

    Error = fabs((xf - xi) * (xf - xi) * (xf - xi) * (xf - xi) * (xf - xi) * GoldenSearch(xi, xf, 100, true) / (180 * nintervals * nintervals * nintervals * nintervals));
    //se alguem mudar isto para pow's
    //saio do grupo
    // OBG
}

double IntegDeriv::FirstDerivativeFowardDifference(double x)
{
    double h = 1e-8;
    return (F(x + h) - F(x)) / h;
}
double IntegDeriv::FirstDerivativeCentralDifference(double x)
{
    double h = 1e-5;
    return (F(x + h) - F(x - h)) / (2 * h);
}
double IntegDeriv::FirstDerivativeFivePoint(double x)
{
    double h = 1e-3;
    return ((F(x - 2 * h) + 8 * F(x + h)) - (8 * F(x - h) + F(x + 2 * h))) / (12 * h);
}

double IntegDeriv::SecondDerivativeFivePoint(double x)
{
    double h = 1e-3;
    return (-F(x - 2 * h) + 16 * F(x - h) - 30 * F(x) + 16 * F(x + h) - F(x + 2 * h)) / (12 * h * h);
}

double IntegDeriv::FourthDerivative(double x)
{

    double h = 1e-3;
    return (-F(x + 3 * h) + 12 * F(x + 2 * h) - 39 * F(x + h) + 56 * F(x) - 39 * F(x - h) + 12 * F(x - 2 * h) - F(x - 3 * h)) / (6 * h * h * h * h);
}

void IntegDeriv::MonteCarlo(double xi, double xf, int ntrials, double &value, double &error)
{

    TRandom *gRandom = new TRandom3(0);
    double sumf = 0;
    double sumsquared = 0;

    for (int i = 1; i <= ntrials; ++i)
    {
        double trial = F(xi + (xf - xi) * gRandom->Rndm());
        sumf = sumf + trial;
        sumsquared = sumsquared + trial * trial;
    }

    error = (xf - xi) / (sqrt(ntrials)) * sqrt(sumsquared / ntrials - (sumf / ntrials) * (sumf / ntrials));

    value = (xf - xi) * sumf / ntrials;

    delete gRandom;
}

void IntegDeriv::MonteCarloNeumann(double xi, double xf, int ntrials, double &value, double &error)
{

    TRandom3 *gRandom = new TRandom3(0);
    double h = 1e-6;
    double max = 0;
    double min = 0;
    double Dx = xf - xi;
    double v;
    int Nrn = 0, nn = 0, Nrp = 0, np = 0;
    bool flag = 0;

    for (int j = 0; (double)j * h < Dx; j++)
    {
        v = F(xi + j * h);
        if (v > max)
            max = v;
        if (v < 0)
            flag = 1;
        if (v < min)
            min = v;
    }
    if (flag == 1)
        std::cout << "Von Neumann method unreliable because the function takes negative values in the given interval" << std::endl;

    for (int i = 1; i <= ntrials; ++i)
    {
        double xrand = xi + (xf - xi) * gRandom->Rndm();
        double yrand = min + (max - min) * gRandom->Rndm();
        if (yrand < 0)
        {
            if (yrand > F(xrand))
                ++Nrn;
            ++nn;
        }
        else if (yrand > 0)
        {
            if (yrand < F(xrand))
                ++Nrp;
            ++np;
        }
    }
    if (min < 0)
    {
        if (max > 0)
        {
            value = (xf - xi) * (max * Nrp / np + min * Nrn / nn);

            error = (xf - xi) * (max - min) * sqrt((Nrp + Nrn) * (1 - ((Nrp + Nrn) / ntrials))) / ntrials; ///////////////////////esta variancia esta mt provavelmente mal calculada
        }
        else
        {
            value = (xf - xi) * (min * Nrn / nn);

            error = (xf - xi) * (-min) * sqrt((Nrn) * (1 - ((Nrn) / nn))) / nn;
        }
    }
    else //True Von Neumann
    {
        value = (xf - xi) * (max * Nrp / np);

        error = (xf - xi) * (max)*sqrt((Nrp) * (1 - ((Nrp) / np))) / np;
    }

    delete gRandom;
}

double IntegDeriv::myPDF(double x, double xi, double xf)
//Function to integrate
{
    return (5. / (pow(xf, 5) - pow(xi, 5))) * x * x * x * x;
}

double IntegDeriv::InverseCumulative(double x, double xi, double xf)
{

    return pow((pow(xf, 5) - pow(xi, 5)) * x + pow(xi, 5), 0.2); //havia aqui um erro, faltava somar pow(xi, 5), so nao demos conta antes pq o xi=0 nos casos q testamos lol
}

void IntegDeriv::MonteCarloImportanceSampling(double xi, double xf, int ntrials, double &value, double &error, bool usemypdf, std::function<double(double)> px, std::function<double(double)> xy)
//Function to execute Monte Carlo integration on predefined function
{
    //Random number generator to generate samples from the companion distribution
    TRandom3 *gRandom = new TRandom3();
    gRandom->SetSeed(0);
    //gRandom->Uniform(0, 1);
    double randNum;
    double weight, xgen;
    double sumratio = 0;
    double totalSumSquared = 0;

    int iter = 0;

    while (iter < ntrials - 1)
    {
        randNum = gRandom->Rndm();

        if (usemypdf)
            xgen = InverseCumulative(randNum, xi, xf);

        else
            xgen = xy(randNum);

        if (xgen != 0)
        {
            if (usemypdf)
                weight = F(xgen) / myPDF(xgen, xi, xf);

            else
                weight = F(xgen) / px(xgen);

            sumratio += weight;

            totalSumSquared += weight * weight;

            iter++;
        }
    }

    value = sumratio / ntrials;

    error = (xf - xi) / (sqrt(ntrials)) * sqrt(totalSumSquared / ntrials - (sumratio / ntrials) * (sumratio / ntrials));

    delete gRandom;
}