#ifndef __IntegDeriv__
#define __IntegDeriv__

#include "Functor.h"
#include <TRandom3.h>
#include <ctime>

class IntegDeriv
{
public:
    IntegDeriv(Functor &);
    ~IntegDeriv() = default;

    // integration methods (retornam o erro pela formula da truncaçao)
    void TrapezoidalRule(double xi, double xf, double &Integral, double &Error, double tol=0.00001);
    void SimpsonRule(double xi, double xf, double &Integral, double &Error, int nintervals=100);
    

    // procura pelos maximos
    double GoldenSearch(double xi, double xf, int intervals, bool D); //D=0 segunda derivada, D=1 4a derivada
    
    //derivative methods
    double FirstDerivativeFowardDifference(double x);
    double FirstDerivativeCentralDifference(double x);
    double FirstDerivativeFivePoint(double x);
    double SecondDerivativeFivePoint(double x);
    double FourthDerivative(double x);

    // monte carlo integration methods
    void MonteCarlo(double xi, double xf, int ntrials, double &value, double &error);
    void MonteCarloNeumann(double xi, double xf, int ntrials, double &value, double &error);//only works for positive functions in the given interval
    double myPDF(double x, double xi, double xf);
    double InverseCumulative(double, double xi, double xf);
    void MonteCarloImportanceSampling(double xi, double xf, int ntrials, double &value, double &error, bool usemypdf=1, std::function<double(double)> px=0, std::function<double(double)> xy=0);
    //PDF proporcional a x^4, so e bom para funcoes crescentes e que para usar e necessario que a=1

private:
    Functor &F;
};
#endif