#include "DerivatorIntegratorND.h"

IntegDerivND::IntegDerivND(FunctorND &function) : FND(function) { ; }

/* integration methods
void IntegDerivND::SimpsonRule(double &xi, double &xf, double &Integral, double &Error, int nintervals = 100);
void IntegDerivND::SimpsonRuleND(double *xi, double *xf, double &Integral, double &Error, int nintervals = 100);
*/
// monte carlo integration methods
void IntegDerivND::MonteCarlo(std::vector<double> xi, std::vector<double> xf, int ntrials, double &value, double &error)
{
    TRandom *gRandom = new TRandom3(0);
    double trial = 0;
    double sumf = 0;
    double sumsquared = 0;
    int n = FND.GetDimension();
    double x[n];
    for (int j = 0; j < n; j++)
        x[j] = 0;
    
    for (int i = 1; i <= ntrials; ++i)
    {
        for (int j = 0; j < n; j++)
        {
            x[j] = xi[j] + (xf[j] - xi[j]) * gRandom->Rndm();
        }
        
        trial = FND(x);
        sumf = sumf + trial;
        sumsquared = sumsquared + trial * trial;
    }
    value = 1;
    for (int j = 0; j < n; j++)
    {
        value = value * (xf[j] - xi[j]);
    }
    error = value / (sqrt(ntrials)) * sqrt(sumsquared / ntrials - (sumf / ntrials) * (sumf / ntrials));
    value = value * sumf / ntrials;
    delete gRandom;
}
/*
void IntegDerivND::MonteCarloImportanceSampling(double *xi, double *xf, int ntrials, double &value, double &error, std::function<double(double *)> px = 0, std::function<double(double *)> xy = 0);
*/