#include "EqSolver.h"
#include "FCMatrix.h"
#include <iostream>
#include <vector>
#include <utility>
using namespace std;

EqSolver::EqSolver(Matrix<double, Dynamic, Dynamic> &Coef, Matrix<double, Dynamic, 1> &Cons)
{
  M = Coef;
  b = Cons;
};

EqSolver::EqSolver(const vector<double>& a, const vector<double>& d, const vector<double>& c, Matrix<double,Dynamic,1>& Cons) : under(a), diagonal(d), upper(c), b(Cons) {}


void EqSolver::teste()
{
  cout << M << endl;
  cout << "\n"
       << b << endl;
};

Matrix<double, Dynamic, 1> EqSolver::GaussSolver(bool pivot)
{ 
  vector<int> index;
  FCMatrixAlgo Gauss;
  int i, n = b.rows(); index.resize(n);
  for(i=0; i<n; ++i) { index[i]=i;}

  if (pivot == 1)
    Gauss.GaussEliminationPivot(M, b, index);
  else
    Gauss.GaussElimination(M, b);
  Matrix<double, Dynamic, 1> sol, sol2;
  sol.resize(n, 1); sol2.resize(n,1);
  
 
  
  for (int i = n - 1; i >= 0; i--)
  {
    double sumX = 0;
    // scan values of a row from diagonal to the right
    if(i != (n-1)){
    for (int j = i + 1; j < n; j++)
    {
      sumX += sol[index[j]]* M(index[i], j);
    }}
    // compute solution values
    sol[index[i]] = (b[index[i]] - sumX) / M(index[i], i);
    sol2[i]=sol[index[i]];
  }
  return sol2;
};

const Matrix<double, Dynamic, 1> &EqSolver::LUSolver()
{
  FCMatrixAlgo LU; int n = b.rows();
  Matrix<double, Dynamic, Dynamic> L = LU.LUDoolittle(M).first;
  Matrix<double, Dynamic, Dynamic> U = LU.LUDoolittle(M).second;
  Matrix<double, Dynamic, 1> y(b);
  Matrix<double, Dynamic, 1> &sol(b);

  for (int k = 0; k < n; k++)
  {
    double sumC = 0.;
    for (int i = 0; i < k; i++)
    {
      sumC += y[i] * L(k, i);
    }
    y[k] = b[k] - sumC;
  }

  for (int k = n - 1; k >= 0; k--)
  {
    double sumC = 0.;
    for (int i = k + 1; i < n; i++)
    {
      sumC += sol[i] * U(k, i);
    }
    sol[k] = (y[k] - sumC) / U(k, k);
  }

  return sol;
};

const Matrix<double, Dynamic, 1>& EqSolver::BandedSolver() {

  int i; 
  int n = b.rows(); 
  
  vector<double>  newd(n);       //n
  vector<double>  newunder(n-1); //n-1
 

  for(i=0; i<n; ++i) {newd[i]= diagonal[i]; }
  for(i=0; i<n-1; ++i) {newunder[i]= 0; }

  
  for(i = 0; i<n-1; ++i){

   newunder[i] = under[i]/newd[i];
   newd[i+1] = diagonal[i+1] - newunder[i]*upper[i];
        
  }

  Matrix<double, Dynamic, 1> y(b); y.resize(n, 1);
  Matrix<double, Dynamic, 1> &sol(b); sol.resize(n,1);


  for(i = 1; i<n; ++i){

    y[i] = b[i] - newunder[i-1]*y[i-1];
    
  }

  
  sol[n-1]=y[n-1]/newd[n-1];

  for(i = n-2; i>=0; --i) {     

    sol[i] = (y[i] - upper[i]*sol[i+1])/newd[i];  

  }

  return sol;
    

}

void EqSolver::IterativeJacobiSolver(Eigen::Matrix<double, Eigen::Dynamic, 1> &x, int itmax, double tol)
{
  int n = b.rows();
  bool btol = false;
  int it = 0;
  Eigen::Matrix<double, Eigen::Dynamic, 1> x_aux;
  while (!btol && (it++ < itmax))
  {

    x_aux = x;
    for (int i = 0; i < n; i++)
    {
      x[i] = 0;
      for (int j = 0; j < n; j++)
      {
        if (i != j)
          x[i] = x[i] + M(i, j) * x_aux[j];
      }

      x[i] = (b[i] - x[i]) / M(i, i);

      // guarantee that all vector entries are converging equally
      if (fabs(x[i] - x_aux[i]) < tol)
        btol = true;
      else
        btol = false;
    }
  }
}

void EqSolver::IterativeGaussSeidelSolver(Eigen::Matrix<double, Eigen::Dynamic, 1> &x, int itmax, double tol)
{
  int n = b.rows();
  bool btol = false;
  int it = 0;
  Eigen::Matrix<double, Eigen::Dynamic, 1> x_aux;

  while (!btol && (it++ < 1000))
  {
    x_aux = x;
    for (int i = 0; i < n; i++)
    {
      x[i] = 0.;
      for (int j = 0; j < n; j++)
        if (i != j)
          x[i] += -M(i, j) * x[j];
      x[i] += b[i];
      x[i] /= M(i, i);
      //guarantee that all vector entries are converging equally
      if (fabs(x[i] - x_aux[i]) < tol)
        btol = true;
      else
        btol = false;
    }
  }
}
