#include "Interpolator.h"

// constructors, destructor
Interpolator::Interpolator(int N, double *x, double *y) : DataPoints(N, x, y) { Init(); }

Interpolator::Interpolator(const std::vector<std::pair<double, double>> &x) : DataPoints(x) { Init(); }

Interpolator::Interpolator(const std::vector<double> &x, const std::vector<double> &y) : DataPoints(x, y) { Init(); }

Interpolator::~Interpolator()
{
    std::cout << "Interpolator Destructor called" << std::endl;
    //P.clear();
    MI.clear();
}

void Interpolator::Init()
{
    Kappa.resize(P.size(), 1);
    SplineKCalc();
    //Create the lambda function for the Lagrange method and store the TF1
    auto l1 = [this](double *x, double *par)
    { return this->InterpolateLagrange(x[0]); };
    TF1 *L = new TF1("Lagrange Interpolator", l1, floor(P[0].first), ceil(P[P.size() - 1].first), 0);
    MI.insert({"Lagrange", L});
    //Create the lambda function for the Newton method and store the TF1
    auto l2 = [this](double *x, double *par)
    { return this->InterpolateNewton(x[0]); };
    TF1 *Newt = new TF1("Newton Interpolator", l2, floor(P[0].first), ceil(P[P.size() - 1].first), 0);
    MI.insert({"Newton", Newt});
    //Create the lambda function for the Spline3 method and store the TF1
    auto l3 = [this](double *x, double *par)
    { return this->InterpolateSpline3(x[0]); };
    TF1 *S = new TF1("Spline3 Interpolator", l3, floor(P[0].first), ceil(P[P.size() - 1].first), 0);
    MI.insert({"Spline3", S});
}
// Lagrange interpolation
double Interpolator::InterpolateLagrange(double fx)
{
    double fly = 0.0; //The value of the function at fx
    for (const auto &p : P)
    {
        double li = 1.0;
        for (const auto &s : P)
        {
            if (s.first != p.first)
            {
                li *= ((fx - s.first) / (p.first - s.first)); //Calulate the various l for the lagrange method
            }
        }
        fly += p.second * li; //Calculate fy
    }
    return fly;
}

// Newton interpolation
double Interpolator::InterpolateNewton(double x)
{
    int N = P.size();
    double a[N];
    double X[N];
    int i = 0;
    for (const auto &p : P) //Store de points on two C arrays
    {
        X[i] = p.first;
        a[i] = p.second;
        i++;
    }

    for (int k = 1; k < N; k++)
    {
        for (i = k; i < N; i++)
        {
            a[i] = (a[i] - a[k - 1]) / (X[i] - X[k - 1]); //Method calculations
        }
    }

    double Poly = a[N - 1];
    for (int k = 1; k < N; k++)
    {
        Poly = a[N - 1 - k] + (x - X[N - 1 - k]) * Poly; // Use the array a for calculation of Poly
    }

    return Poly;
}

// spline3
double Interpolator::InterpolateSpline3(double fx)
{
    int i = 0;
    while (1) //Identify the position of fx in our set of points
    {
        if (fx < P[i].first && i == 0)
        {
            break;
        }
        else
        {
            if (fx > P[i].first && i == P.size() - 1)
            {
                --i;
                break;
            }
            else
            {
                if (fx < P[i].first && i > 0)
                {
                    --i;
                    break;
                }

                else
                {
                    ++i;
                }
            }
        }
    }
    //Calculate the value of the interpolation using the K matrix
    return (Kappa[i] / 6) * ((pow(fx - P[i + 1].first, 3)) / (P[i].first - P[i + 1].first) - (fx - P[i + 1].first) * (P[i].first - P[i + 1].first)) -
           (Kappa[i + 1] / 6) * ((pow(fx - P[i].first, 3)) / (P[i].first - P[i + 1].first) - (fx - P[i].first) * (P[i].first - P[i + 1].first)) +
           (P[i].second * (fx - P[i + 1].first) - P[i + 1].second * (fx - P[i].first)) / (P[i].first - P[i + 1].first);
}

// draw points and function
void Interpolator::Draw(std::string s)
{
    if (s == "Lagrange" || s == "Spline3" || s == "Newton" || s == "All")
    {

        TCanvas *c1 = new TCanvas("c1", "Grafico", 550, 100, 800, 600);
        TGraph *gr1 = DataPoints::GetGraph();                //Return the graph for the points
        TLegend *legend = new TLegend(0.43, 0.7, 0.67, 0.9); //Create Label
        //Label settings
        legend->SetHeader("Legend", "C");
        legend->AddEntry(gr1, "Given points");
        //Set title
        gr1->SetTitle("Pendulum L=4.0 [m] | Initial Cond.: #theta_{0}= 1.4 [rad] | v_{0}= 0.0 [rad/sec];Angle(#theta) [rad];Time [sec]");

        //Para depois usar a classe geralmente temos de meter um titulo geral, como:
        //gr1->SetTitle("Data points and respective Interpolation; x; y");
        //gr1 settings
        gr1->SetMarkerColor(kBlue);
        gr1->SetMarkerStyle(kFullCircle);

        //Isto foi so para o grafico ficar como o anterior mas deve ser comentado qnd for para usar geralmente
        gr1->SetMinimum(-3.);
        gr1->SetMaximum(3.);

        //Oppening an interface where the graph will be shown
        //TApplication app("app", nullptr, nullptr);
        gr1->Draw("AP");
        if (s != "All") //Draw only one
        {
            TF1 *FUNCAO = (MI.find(s)->second); //Store TF1
            const char *Titulo = s.c_str();     //Make the title a c array of char
            //TF1 settings
            FUNCAO->SetLineColor(kRed);
            FUNCAO->Draw("Same");
            //Label settings
            legend->AddEntry(FUNCAO, Titulo);
            legend->Draw();
        }

        else //Draw all at the same time
        {
            //Get all interpolations
            TF1 *FuncS = (MI.find("Spline3")->second);
            TF1 *FuncN = (MI.find("Newton")->second);
            TF1 *FuncL = (MI.find("Lagrange")->second);
            //Settings for all the interpolations
            FuncN->SetLineColor(kRed);
            FuncS->SetLineColor(kGreen);
            FuncL->SetLineColor(kBlue);
            FuncN->SetLineWidth(3);
            FuncN->Draw("Same");
            FuncS->Draw("Same");
            FuncL->Draw("Same");
            //Settings for the label
            legend->AddEntry(FuncS, "Spline3 Interpolation");
            legend->AddEntry(FuncN, "Newton Interpolation");
            legend->AddEntry(FuncL, "Lagrange Interpolation");
            legend->Draw();
        }

        //Update and save canvas
        c1->Modified();
        c1->Update();
        c1->SaveAs("HW3.png");

        //To end program after the app has benn closed
        //TRootCanvas *rc = (TRootCanvas *)c1->GetCanvasImp();
        //rc->Connect("CloseWindow()", "TApplication", gApplication, "Terminate()");

        //Running the application
        //app.Run();

        //Deleting allocated space
        delete c1;
        delete gr1;
        delete legend;
    }

    else
    {
        //If the string input is not valid
        std::cout << "ERRO: Nao ha nenhuma interpolacao com esse nome\n Nomes possiveis: Lagrange, Newton, Spline3" << std::endl;

        DataPoints::Draw();
    }

} // s="Lagrange", "Newton", "Spline3"

// getters

TF1 *Interpolator::GetFunction(std::string s)
{

    if (s == "Newton" | s == "Spline3" | s == "Lagrange")
    {
        return MI.find(s)->second; //Return the corresponding TF1
    }

    else
    {
        std::cout << "ERRO: Nao ha nenhuma interpolacao com esse nome\n Nomes possiveis: Lagrange, Newton, Spline3" << std::endl;
        //return MI.find(s)->second;
        return 0; //No return?
    }
} // s="Newton", "Spline3"\\\ Será que é TF1& ou TF1*?

void Interpolator::SplineKCalc() //Calculate K for the Spline3 Method
{
    //Change the size of the matrix
    int n = P.size() - 2;
    Matrix<double, Dynamic, 1> Ktemp;
    Ktemp.resize(n, 1);
    Matrix<double, Dynamic, Dynamic> m;
    m.resize(n, n);
    Matrix<double, Dynamic, 1> b;
    b.resize(n, 1);

    for (int i = 0; i < n; ++i)
    {

        for (int j = 0; j < n; ++j)
        {
            m(i, j) = 0; //Intialize the matrix with zero
        }
        //fill the right places at the matrix
        if (i == 0)
        {
            m(i, 0) = 2 * (P[0].first - P[2].first);

            if (n > 1)
            {
                m(i, 1) = (P[1].first - P[2].first);
            }
        }

        if (i != 0)
        {
            if (n > 1 && i == n - 1)
            {
                m(i, i) = 2 * (P[i].first - P[i + 2].first);
                m(i, i - 1) = (P[i].first - P[i + 1].first);
            }

            else
            {
                m(i, i) = 2 * (P[i].first - P[i + 2].first);
                m(i, i + 1) = P[i + 1].first - P[i + 2].first;
                m(i, i - 1) = P[i].first - P[i + 1].first;
            }
        }
    }
    //Fill column matrix b
    for (int i = 0; i < n; ++i)
    {
        b(i, 0) = 6 * ((P[i].second - P[i + 1].second) / (P[i].first - P[i + 1].first) - (P[i + 1].second - P[i + 2].second) / (P[i + 1].first - P[i + 2].first));
    }
    //Solve the linear system with EqSolver and FCMatrixAlgo
    FCMatrixAlgo Resolve;
    EqSolver Equ(m, b);
    Ktemp = Equ.GaussSolver(1); //Temporary K
    Kappa[0] = 0;
    Kappa[P.size() - 1] = 0;
    for (int i = 1; i <= n; ++i)
    {
        Kappa[i] = Ktemp[i - 1]; //Get final K
    }
}