#ifndef __INTERPOLATOR__
#define __INTERPOLATOR__

#include "DataPoints.h"
#include "EqSolver.h"
#include "FCMatrix.h"
#include "TF1.h"
#include "TLegend.h"
#include <string>

class Interpolator : public DataPoints
{
public:
    // constructors, destructor
    Interpolator(int N = 0, double *x = NULL, double *y = NULL); // build DataPoints from C-arrays of x and y values
    Interpolator(const std::vector<std::pair<double, double>> &); //Build DataPoints from a vector of pairs
    Interpolator(const std::vector<double> &x, const std::vector<double> &y); //Builda DataPoints from two vectors
    ~Interpolator(); //Destructor
    // interpolation methods
    void Init(); //Auxiliar function
    double InterpolateLagrange(double); // Lagrange interpolation
    double InterpolateNewton(double);   // Newton interpolation
    double InterpolateSpline3(double);  // spline3
    // draw points and function
    void Draw(std::string s); // s="Lagrange", "Neville", "Spline3"
    // getters
    TF1 *GetFunction(std::string s); // s="Neville", "Spline3"

    //for K matrix
    void SplineKCalc();

private:
    std::map<std::string, TF1 *> MI; // key="Lagrange", "Neville", "Spline3"
    Matrix<double, Dynamic, 1> Kappa; //Matrix of the coefficients for the Spline3 Method
};
#endif



