#ifndef __ODESOLVER__
#define __ODESOLVER__

#include "ODEpoint.h"
#include <map>
#include <functional>
#include <cmath>
#include "TCanvas.h"
#include "TGraph.h"
#include "TLegend.h"
#include "TAxis.h"
#include "TMultiGraph.h"
#include "TGraph2D.h"

class ODESolver
{
public:
    ODESolver();
    ODESolver(const int, const Xvar &, std::vector<std::function<double(ODEpoint &)>> &, std::vector<double> &, double, const bool = 1);
    ODESolver(const int, const Xvar &, std::vector<std::function<double(ODEpoint &)>> &, const bool = 1);
    ODESolver(const int, const std::initializer_list<double> &, std::vector<std::function<double(ODEpoint &)>> &, std::vector<double> &, double, const bool = 1);
    ODESolver(const int, const std::initializer_list<double> &, std::vector<std::function<double(ODEpoint &)>> &, const bool = 1);

    ~ODESolver();
    // solvers
    const std::vector<ODEpoint> &VerletSolver(double step = 1E-3, double Time = 10);
    const std::vector<ODEpoint> &TrapezoidalSolver(double step = 1E-3, double Time = 10);
    const std::vector<ODEpoint> &EulerSolver(double step = 1E-3, double Time = 10);
    const std::vector<ODEpoint> &RungeKutta4(double step = 1E-3, double Time = 10);
    const std::vector<ODEpoint> &RungeKutta2(double step = 1E-3, double Time = 10);
    const std::vector<ODEpoint> &LeapFrog(double step = 1E-3, double Time = 10);

    //setters
    void SetXvar(const Xvar&);

    // dimensionalizations
    const std::vector<ODEpoint> &Dimensionalize(std::string s = "Runge-Kutta");

    // graphics
    void Draw(std::string s = "verlet", double ti = 0., double tf = 10., bool PhaseDiagram = 0, bool Dimensionalized = 0);

private:
    //n-order differential equation written in the following order: the last equation of the system (f[n-1]) is the n-order differential equation
    const bool onequation;

    //factors to diminish numerical errors, factor[0]->f[0], ..., factor[n]->f[n]
    std::vector<double> factor;
    double factort;

    //number of equations
    const int Ndim;

    // initial conditions: angle (rad), angular velocity (rad/s)
    Xvar X0;

    // solutions
    std::map<std::string, std::vector<ODEpoint>> MS;  // key: "verlet", "euler", "trapezoidal", "runge-kutta"
    std::map<std::string, std::vector<ODEpoint>> MDS; // dimensionalized

    // lambda functions needed
    std::vector<std::function<double(ODEpoint &)>> f; // f_theta(t,theta,omega), f_omega(t,theta,omega)
};
#endif