#ifndef __ODEPOINT__
#define __ODEPOINT__

#include <iostream>
#include <vector>
#include <initializer_list>
#include <algorithm>

class Xvar
{
public:
    Xvar() = default; // Default constructor
    Xvar(int);                 // Number of dependent variables
    Xvar(std::vector<double>); // Passing vector
    // Using initializer list to build object: X({1,2})
    Xvar(const std::initializer_list<double> &v);
    ~Xvar(); //Default destrucotr

    Xvar(const Xvar &);                          // copy constructor
    Xvar& operator=(const Xvar &);               // assignment operator
    Xvar operator+(const Xvar &);                // operator+
    double& operator[](int);                     // X[i] 

    friend Xvar operator*(double, const Xvar &); // scalar*Xvar
    friend std::ostream &operator<<(std::ostream &, const Xvar &); // Output operator

    std::vector<double>& X(); // accessor to x

protected:
    std::vector<double> x; //Vector that stores our variables
};

///////////////////////////////////////////////////////////////////////////////////////////

class ODEpoint : public Xvar
{
public:
    ODEpoint() : t(-1), Xvar() {;}// Default Constructor
    //Constructors
    ODEpoint(double t_, Xvar a_) : t(t_), Xvar(a_) { ; }
    ODEpoint(double t_, const std::vector<double> &v) : t(t_), Xvar(v) { ; }
    ODEpoint(double t_, const std::initializer_list<double> &v) : t(t_), Xvar(v) { ; }
    ~ODEpoint(); //Destructor

    //Various ways to store an ODEpoint
    void SetODEpoint(double t_, Xvar &p);
    void SetODEpoint(double t_, const std::initializer_list<double> &v);
    void SetODEpoint(double t_, std::vector<double> v);
    
    double &T() { return t; } // accessor to time
    
private:
    double t; // time
};
#endif