#include "Roots.h"
using namespace std;

pair<double, double> RootSolver::bissection(double xi, double xf)
{
   int i = 0;
   if (funcao(xi) * funcao(xf) >= 0)
   {
      cout << "The given interval does not contain a root, please insert another interval\n";
      return make_pair(123456789, 123456789);
   }

   double c = xi;
   while (abs(xf - xi) >= 0.001)
   {
      c = (xi + xf) / 2;

      if (funcao(c) == 0.0)
         break;

      else if (funcao(c) * funcao(xi) < 0)
         xf = c;

      else
         xi = c;
      ++i;
   }
   return make_pair(c, i);
}

RootSolver::~RootSolver() {}

pair<double, double> RootSolver::regfalsi(double xi, double xf)
{
   double c = xi;
   int i = 0;

   if (funcao(xi) * funcao(xf) >= 0)
   {
      cout << "The given interval does not contain a root, please insert another interval\n";
      return make_pair(123456789, 123456789);
   }

   while (abs(xf - xi) >= 0.001)
   {
      c = xi - funcao(xi) * (xf - xi) / (funcao(xf) - funcao(xi));

      if (funcao(c) == 0.0)
         break;

      else if (funcao(c) * funcao(xi) < 0)
         xf = c;

      else
         xi = c;

      ++i;
   }
   return make_pair(c, i);
}

pair<double, double> RootSolver::secant(double xi, double xf)
{
   double c = (xi + xf) / 2;
   int i = 0;
   double extra1 = xi, extra2;

   while (abs(funcao(c)) >= 0.00001)
   {
      extra2 = c - funcao(c) * ((c - extra1) / (funcao(c) - funcao(extra1)));
      extra1 = c;
      c = extra2;
      ++i;
   }
   return make_pair(c, i);
}

pair<double, double> RootSolver::Newton(double xi, double xf)
{
   IntegDeriv Aux(funcao);
   double c = (xi + xf) / 2;
   int i = 0;

   while (abs(funcao(c)) >= 0.00001)
   {
      c = c - funcao(c) / (Aux.FirstDerivativeFivePoint(c));
      ++i;
   }
   return make_pair(c, i);
}

pair<double, double> RootSolver::NewtonAnalytical(double xi, double xf, Functor &derivada)
{
   double c = (xi + xf) / 2;
   int i = 0;

   while (abs(funcao(c)) >= 0.00001)
   {
      c = c - funcao(c) / derivada(c);
      ++i;
   }
   return make_pair(c, i);
}