#include "Rwalk1D.h"

Rwalk1D::Rwalk1D(int nb, double xi, double Pl, double Pr, double dT, double dX) : N(nb), x0(xi), pL(Pl), pR(Pr), dt(dT), dx(dX)
{
    if (pL + pR > 1 || pL < 0 || pR < 0)
    {
        throw std::runtime_error("Failed to construct object: Probabilities not well defined"); //See if the probabilities make sense
    }
    for (int i = 1; i <= N; i++)
    {
        mT[i]; //Initialize de map
    }
}

Rwalk1D::~Rwalk1D()//Define the destructor
{
    std::cout << "[Rwalk1D destructor called]" << std::endl;
    mT.clear();
}

void Rwalk1D::Run(int nsteps) //Create the n random trajectory
{
    TRandom3 *gRandom = new TRandom3(0); //Random Number generator from Root
    for (int j = 1; j <= N; j++)
    {
        mT[j].push_back(x0);
        double xstep, x=x0;

        for (int i = 0; i < nsteps; i++)
        {
            xstep = gRandom->Rndm();
            if (xstep <= pL)//left
            {
                x-=dx;
            }
            else if(xstep<=(pL+pR))//right
            {
                x+=dx;
            }
            else//same
            x=x;
            
            mT[j].push_back(x);
        }
    }
    delete gRandom;
}

const std::vector<double> &Rwalk1D::GetTrajectory(int n)
{
    return mT.find(n)->second; //Return the trajectory
}
double Rwalk1D::GetTimeStep()
{
    return dt; //Return the time step
}
double Rwalk1D::GetSpaceStep()
{
    return dx; //Return the space step
}
std::ostream &operator<<(std::ostream &os, const Rwalk1D &D)
{
    os << "N= " << D.N << " | x0= " << D.x0 << " | pL= " << D.pL << " | pR= " << D.pR << " | dt= " << D.dt << " | dx= " << D.dx << std::endl; //Write the proprierties of the random walk

    return os;
}