#ifndef __RWALK1D__
#define __RWALK1D__

#include <iostream>
#include <algorithm>
#include <vector>
#include <map>
#include "TH1D.h"
#include "TRandom3.h"

class Rwalk1D
{
public:
    Rwalk1D(int nb = 1, double xi = 0.,       // N=nb of particles, x=x(0)
            double Pl = 0.5, double Pr = 0.5, // Probabilities Left Right
            double dT = 1, double dX = 1      // Time and Space steps
    );
    ~Rwalk1D();
    // particle simulation
    void Run(int nsteps); // number of steps

    // getters
    const std::vector<double>& GetTrajectory(int n = 0); // particle index (0,1,2,...)
    double GetTimeStep();
    double GetSpaceStep();

    friend std::ostream &operator<<(std::ostream &os, const Rwalk1D &D); //Output operator

private:
    double x0;     // init coo
    int N;         // number of particles
    double pL, pR; // probabilities (left, same, righ)
    double dt, dx; // steps (time, space)

    // store trajectories for every particle (key=particle index)
    std::map<int, std::vector<double>> mT;

    // store histograms with particle positions (key=step number)
    std::map<int, std::vector<TH1D *>> mH;
};

#endif