#ifndef __PENDULUM__
#define __PENDULUM__

#include "ODEpoint.h"
#include <map>
#include <functional>
#include <cmath>
#include "TCanvas.h"
#include "TGraph.h"
#include "TLegend.h"
#include "TAxis.h"
#include "TMultiGraph.h"
class pendulum
{
public:
    pendulum();
    pendulum(double, const Xvar &);
    pendulum(double, const std::initializer_list<double> &); // pendulum(10, {80, 0})
    // length in meters, angles in degrees, velocity degrees/sec
    pendulum(double length, double theta_0 = 80, double theta_vel_0 = 0);
    ~pendulum() = default;
    // solvers
    const std::vector<ODEpoint> &VerletSolver(double step=1E-3, double Time=10);
    const std::vector<ODEpoint> &TrapezoidalSolver(double step=1E-3, double Time=10);
    const std::vector<ODEpoint> &RungeKutta4(double step=1E-3, double Time=10);
    const std::vector<ODEpoint> &EulerSolver(double step=1E-3, double Time=10);
    // graphics
    void Draw(std::string s = "verlet", double ti = 0., double tf = 10.);
    void Init();

private:
    double L; // length (m)
    Xvar X0;  // initial conditions: angle (rad), angular velocity (rad/s)
    // solutions
    std::map<std::string, std::vector<ODEpoint>> MS; // key: "verlet", "euler", "trapezoidal", "runge-kutta"
    // lambda functions needed
    std::function<double(ODEpoint &)> f[2]; // f_theta(t,theta,omega), f_omega(t,theta,omega)
};

#endif